@@ -1,10 +1,10 @@
 import functools
-import inspect
+import time
 from collections import deque
 from contextvars import ContextVar
 from copy import copy
 
-from .selector import Element, MatchFunction, to_pattern
+from .selector import Element, MatchFunction, select
 from .selfless import Override, Selfless, choose, override
 from .tags import match_tag
 from .utils import (
@@ -33,11 +33,11 @@ class Frame:
         for cap, varnames in captures.items():
             for v in varnames:
                 self.accumulators.add(v)
-                if acc.rulename == "value" and cap.focus:
+                if acc.varget and cap.focus:
                     self.getters.setdefault(v, []).append((cap, acc))
                 else:
                     self.setters.setdefault(v, []).append((cap, acc))
-        if close_at_exit and acc.rulename == "listeners":
+        if close_at_exit and acc.close:
             self.to_close.append(acc)
 
     def set(self, varname, key, category, value):
@@ -93,22 +93,32 @@ class Capture:
                 f"Multiple values stored for capture `{self.capture}`"
             )
 
-    def acquire(self, varname, value):
+    def accum(self, varname, value):
         assert varname is not None
         self.names.append(varname)
         self.values.append(value)
 
+    def set(self, varname, value):
+        assert varname is not None
+        self.names = [varname]
+        self.values = [value]
+
+    def snapshot(self):
+        cap = Capture(self.element)
+        cap.names = list(self.names)
+        cap.values = list(self.values)
+        return cap
+
     def __str__(self):
         return f"Capture({self.element}, {self.names}, {self.values})"
 
     __repr__ = __str__
 
 
-class Accumulator:
+class BaseAccumulator:
     def __init__(
         self,
         *,
-        rulename,
         parent=None,
         rules=None,
         template=True,
@@ -123,13 +133,23 @@ class Accumulator:
         self.status = ACTIVE
         self.template = template
         self.focus = focus
-        self.rulename = rulename
-        if self.parent is not None:
+        if self.parent is None:
+            self.names = set(pattern.all_captures())
+        else:
+            self.names = self.parent.names
             self.parent.children.append(self)
 
+    def fork(self, focus=True, pattern=None):
+        parent = None if self.template else self
+        return type(self)(
+            parent=parent,
+            rules=self.rules,
+            template=False,
+            pattern=pattern,
+            focus=focus,
+        )
+
     def getcap(self, element):
-        if element.capture is None:
-            return None
         if element.capture not in self.captures:
             cap = Capture(element)
             self.captures[element.capture] = cap
@@ -137,38 +157,7 @@ class Accumulator:
         else:
             return self.captures[element.capture]
 
-    def fail(self):
-        self.status = FAILED
-        for leaf in self.leaves():
-            leaf.status = FAILED
-
-    def varset(self, element, varname, category, value):
-        if (
-            element.value is ABSENT
-            or element.value == value
-            or (
-                isinstance(element.value, MatchFunction)
-                and element.value.fn(value)
-            )
-        ):
-            acc = self.fork(pattern=element) if element.focus else self
-            cap = acc.getcap(element)
-            if cap:
-                cap.acquire(varname, value)
-            if element.focus and self.rulename == "immediate":
-                acc.run_listeners()
-        else:
-            self.fail()
-
-    def varget(self, element, varname, category):
-        cap = Capture(element)
-        self.captures[element.capture] = cap
-        cap.names.append(varname)
-        rval = self.run_value()
-        del self.captures[element.capture]
-        return rval
-
-    def build(self):
+    def build_all(self):
         if self.parent is None:
             return self.captures
         rval = {}
@@ -178,20 +167,16 @@ class Accumulator:
             curr = curr.parent
         return rval
 
-    def run_value(self):
-        rval = ABSENT
-        args = self.build()
-        for fn in self.rules:
-            rval = fn(**args)
-        return rval
+    def build(self):
+        rval = self.build_all()
+        return rval and {k: v for k, v in rval.items() if not k.startswith("/")}
 
-    def run_listeners(self):
-        args = self.build()
-        for fn in self.rules:
-            if set(args) != set(get_names(fn)):
-                return ABSENT
-            else:
-                fn(**args)
+    def check_value(self, evalue, value):
+        return (
+            evalue is ABSENT
+            or evalue == value
+            or (isinstance(evalue, MatchFunction) and evalue.fn(value))
+        )
 
     def leaves(self):
         if isinstance(self.pattern, Element) and self.focus:
@@ -210,51 +195,112 @@ class Accumulator:
                 rval += child._to_merge()
         return rval
 
+    varset = None
+    varget = None
+    close = None
+
+
+class TotalAccumulator(BaseAccumulator):
+    def fail(self):
+        self.status = FAILED
+        for leaf in self.leaves():
+            leaf.status = FAILED
+
+    def varset(self, element, varname, category, value):
+        if self.check_value(element.value, value):
+            acc = self.fork(pattern=element) if element.focus else self
+            cap = acc.getcap(element)
+            cap.accum(varname, value)
+            return acc
+        else:
+            self.fail()
+            return None
+
+    def run(self):
+        args = self.build()
+        for fn in self.rules:
+            if set(args) != self.names:
+                return ABSENT
+            else:
+                fn(**args)
+
     def close(self):
-        assert self.rulename == "listeners"
         if self.status is ACTIVE:
             if self.parent is None:
                 for acc in self._to_merge():
                     self.captures.update(acc.captures)
                 leaves = self.leaves()
                 for leaf in leaves or [self]:
-                    leaf.run_listeners()
+                    leaf.run()
             self.status = COMPLETE
 
-    def fork(self, focus=True, pattern=None):
-        parent = None if self.template else self
-        return Accumulator(
-            parent=parent,
-            rules=self.rules,
-            template=False,
-            pattern=pattern,
-            focus=focus,
-            rulename=self.rulename,
-        )
+
+class ImmediateAccumulator(BaseAccumulator):
+    def build_all(self):
+        rval = super().build_all()
+
+        for cap in rval.values():
+            element = cap.element
+            if element.value is not ABSENT:
+                if not self.check_value(element.value, cap.value):
+                    return None
+
+        rval = {k: cap.snapshot() for k, cap in rval.items()}
+        return rval
+
+    def varset(self, element, varname, category, value):
+        acc = self.fork(pattern=element) if element.focus else self
+        cap = acc.getcap(element)
+        cap.set(varname, value)
+        return acc
+
+    def run(self):
+        rval = ABSENT
+        args = self.build()
+        if args is None:
+            return ABSENT
+        for fn in self.rules:
+            rval = fn(**args)
+        return rval
 
 
-def get_names(fn):
-    if not hasattr(fn, "_ptera_argspec"):
-        spec = inspect.getfullargspec(fn)
-        if spec.args and spec.args[0] == "self":
-            fn._ptera_argspec = spec.args[1:]
-        else:
-            fn._ptera_argspec = spec.args
-    return fn._ptera_argspec
+class SetterAccumulator(ImmediateAccumulator):
+    def varset(self, element, varname, category, value):
+        acc = super().varset(element, varname, category, value)
+        if acc and element.focus:
+            acc.run()
+        return acc
+
+
+class GetterAccumulator(ImmediateAccumulator):
+    def varget(self, element, varname, category):
+        cap = Capture(element)
+        self.captures[element.capture] = cap
+        cap.names.append(varname)
+        rval = self.run()
+        del self.captures[element.capture]
+        return rval
+
+
+accumulator_classes = {
+    "listeners": TotalAccumulator,
+    "immediate": SetterAccumulator,
+    "value": GetterAccumulator,
+}
 
 
 def dict_to_collection(*rulesets):
     tmp = {}
     for rules in rulesets:
         for pattern, triggers in rules.items():
-            pattern = to_pattern(pattern)
+            pattern = select(pattern)
             for name, entries in triggers.items():
                 key = (name, pattern)
                 if not isinstance(entries, (tuple, list)):
                     entries = [entries]
                 for entry in entries:
                     if key not in tmp:
-                        tmp[key] = Accumulator(rulename=name, pattern=pattern)
+                        tmp[key] = accumulator_classes[name](pattern=pattern)
                     acc = tmp[key]
                     acc.rules.append(entry)
     return PatternCollection(
@@ -316,6 +362,8 @@ class PatternCollection:
         to_process = deque(self.patterns)
         while to_process:
             pattern, acc = to_process.pop()
+            if acc.status is FAILED:
+                continue
             if not pattern.immediate:
                 next_patterns.append((pattern, acc))
             cachekey = (fn, pattern)
@@ -479,7 +527,6 @@ class Collector:
             def listener(**kwargs):
                 self.data.append(kwargs)
 
-        listener._ptera_argspec = set(self.pattern.all_captures())
         self._listener = listener
 
     def __iter__(self):
@@ -542,7 +589,7 @@ class Tap:
     hasoutput = True
 
     def __init__(self, selector, mapper=None, immediate=False):
-        self.selector = to_pattern(selector)
+        self.selector = select(selector)
         self.mapper = mapper
         self.immediate = immediate
 
@@ -589,13 +636,6 @@ def _to_plugin(spec, **kwargs):
     return Tap(spec, **kwargs) if isinstance(spec, str) else spec
 
 
-def _collect_plugins(plugins, kwplugins):
-    plugins = {str(i + 1): p for i, p in enumerate(plugins)}
-    plugins.update(kwplugins)
-    plugins = {name: _to_plugin(p) for name, p in plugins.items()}
-    return plugins, any(p.hasoutput for name, p in plugins.items())
-
-
 class Overlay:
     def __init__(self, plugins={}):
         self.plugins = dict(plugins)
@@ -604,14 +644,25 @@ class Overlay:
     def hasoutput(self):
         return any(p.hasoutput for name, p in self.plugins.items())
 
-    def use(self, *plugins, **kwplugins):
-        plugins, _ = _collect_plugins(plugins, kwplugins)
+    def __use(self, plugins, kwplugins, immediate):
+        plugins = {str(i + 1): p for i, p in enumerate(plugins)}
+        plugins.update(kwplugins)
+        plugins = {
+            name: _to_plugin(p, immediate=immediate)
+            for name, p in plugins.items()
+        }
         self.plugins.update(plugins)
         return self
 
+    def use(self, *plugins, **kwplugins):
+        return self.__use(plugins, kwplugins, True)
+
+    def full_tap(self, *plugins, **kwplugins):
+        return self.__use(plugins, kwplugins, False)
+
     def tweak(self, values, priority=2):
         values = {
-            to_pattern(k): lambda __v=v, **_: override(__v, priority)
+            select(k): lambda __v=v, **_: override(__v, priority)
             for k, v in values.items()
         }
         self.plugins[f"#{len(self.plugins)}"] = StateOverlay(values)
@@ -625,7 +676,6 @@ class Overlay:
                     call_with_captures(fn, kwargs, full=full), priority=priority
                 )
 
-            newfn._ptera_argspec = get_names(fn)
             return newfn
 
         values = {k: _wrapfn(v, full=full) for k, v in values.items()}
@@ -634,8 +684,13 @@ class Overlay:
 
     @autocreate
     def using(self, *plugins, **kwplugins):
-        plugins, _ = _collect_plugins(plugins, kwplugins)
-        return Overlay(plugins={**self.plugins, **plugins})
+        ol = Overlay(self.plugins)
+        return ol.use(*plugins, **kwplugins)
+
+    @autocreate
+    def full_tapping(self, *plugins, **kwplugins):
+        ol = Overlay(self.plugins)
+        return ol.full_tap(*plugins, **kwplugins)
 
     @autocreate
     def tweaking(self, values, priority=2):
@@ -736,6 +791,10 @@ class PteraFunction(Selfless):
         self.overlay.use(*plugins, **kwplugins)
         return self
 
+    def full_tap(self, *plugins, **kwplugins):
+        self.overlay.full_tap(*plugins, **kwplugins)
+        return self
+
     def tweak(self, values, priority=2):
         self.overlay.tweak(values, priority=priority)
         return self
@@ -758,6 +817,10 @@ class PteraFunction(Selfless):
         ol = self.overlay.using(*plugins, **kwplugins)
         return self.clone(overlay=ol, return_object=ol.hasoutput)
 
+    def full_tapping(self, *plugins, **kwplugins):
+        ol = self.overlay.full_tapping(*plugins, **kwplugins)
+        return self.clone(overlay=ol, return_object=ol.hasoutput)
+
     def collect(self, query):
         return self.overlay.collect(query)
 
@@ -774,6 +837,7 @@ class PteraFunction(Selfless):
         self.ensure_state()
         with self.overlay as callres:
             with proceed(self):
+                interact("#time", None, None, self, time.time())
                 if self.attachments:
                     for k, v in self.attachments.items():
                         interact(f"#{k}", None, None, self, v)
