@@ -1,11 +1,11 @@
 """Specifications for call paths."""
 
 
-import ast
 import builtins
 import re
 import sys
 from dataclasses import dataclass
+from itertools import count
 
 from . import opparse
 from .tags import Tag
@@ -64,7 +64,7 @@ class Element(metaclass=InternedMC):
         return Element(**args)
 
     def all_captures(self):
-        if self.capture:
+        if self.capture and not self.capture.startswith("/"):
             return {self.capture}
         else:
             return set()
@@ -236,7 +236,7 @@ parser = opparse.Parser(
     lexer=opparse.Lexer(
         {
             r"\s*(?:\bas\b|>>|!+|\[\[|\]\]|[(){}\[\]>:,$=~])?\s*": "OPERATOR",
-            r"[a-zA-Z_0-9#*.]+": "WORD",
+            r"[a-zA-Z_0-9#*.-]+": "WORD",
             r"'[^']*'": "STRING",
         }
     ),
@@ -476,11 +476,13 @@ def make_equals(node, element, value, context, matchfn=False):
         element = evaluate(element, context=context)
     value = value_evaluate(value)
     if matchfn:
-        capture = element.capture
         value = VCall(MatchFunction, (value,))
+    if isinstance(element, Element):
+        capture = element.capture if matchfn else None
+        return element.clone(value=value, capture=capture)
     else:
-        capture = None
-    return element.clone(value=value, capture=capture)
+        new_element = Element(name="#value", value=value, capture=None)
+        return element.clone(captures=element.captures + (new_element,))
 
 
 @evaluate.register_action("_ ~ X")
@@ -516,9 +518,9 @@ class VSymbol(VNode):
     def eval(self, env):
         x = self.value
 
-        if re.match(r"[0-9]+\.[0-9]*", x):
+        if re.match(r"-?[0-9]+\.[0-9]*", x):
             return float(x)
-        elif re.match(r"[0-9]+", x):
+        elif re.match(r"-?[0-9]+", x):
             return int(x)
         elif re.match(r"'[^']*'", x):
             return x[1:-1]
@@ -609,47 +611,15 @@ def parse(x):
     return evaluate(parser(x))
 
 
-_string_cache = {}
-
-
-class _StringFinder(ast.NodeVisitor):
-    def __init__(self):
-        self.strings = {}
-
-    def visit_Str(self, node):  # pragma: no cover
-        # Python <3.8
-        self.strings[node.s] = node.lineno
-
-    def visit_Constant(self, node):  # pragma: no cover
-        # Python >=3.8
-        self.strings[node.value] = node.lineno
-
-
-def _find_string(s, filename):
-    if filename not in _string_cache:
-        with open(filename) as module:
-            tree = ast.parse(module.read(), filename)
-            finder = _StringFinder()
-            finder.visit(tree)
-            _string_cache[filename] = finder.strings
-    return _string_cache[filename].get(s, None)
-
-
-def _find_eval_env(s, fr):
-    ev = None
+def _find_eval_env(s, fr, skip):
     while fr is not None:
-        filename = fr.f_code.co_filename
-        if "/_pytest/" in filename or "/pluggy/" in filename:
-            fr = fr.f_back
-            continue
-        lineno = _find_string(s, filename)
-        if lineno is not None:
-            if ev is not None:
-                if ev[0] != filename:  # pragma: no cover
-                    raise Exception(f"Ambiguous env for selector '{s}'")
-            ev = (filename, fr.f_globals)
+        glb = fr.f_globals
+        name = glb["__name__"]
+        print(name, skip, all(not name.startswith(pfx) for pfx in skip))
+        if all(not name.startswith(pfx) for pfx in skip):
+            return glb
         fr = fr.f_back
-    return ev and ev[1]
+    raise AssertionError("Unreachable outside ptera.")  # pragma: no cover
 
 
 @dataclass(frozen=True)
@@ -657,24 +627,29 @@ class MatchFunction:
     fn: object
 
 
-def _resolve(pattern, env):
+def _resolve(pattern, env, cnt):
     if isinstance(pattern, Call):
-        el = _resolve(pattern.element, env)
+        el = _resolve(pattern.element, env, cnt)
         return pattern.clone(
             element=el,
-            captures=tuple(_resolve(x, env) for x in pattern.captures),
-            children=tuple(_resolve(x, env) for x in pattern.children),
+            captures=tuple(_resolve(x, env, cnt) for x in pattern.captures),
+            children=tuple(_resolve(x, env, cnt) for x in pattern.children),
         )
     elif isinstance(pattern, Element):
         name = _eval(pattern.name, env)
         category = _eval(pattern.category, env)
         value = _eval(pattern.value, env)
+        capture = (
+            f"/{next(cnt)}" if pattern.capture is None else pattern.capture
+        )
         if category is not None and not isinstance(category, Tag):
             raise TypeError(f"A pattern can only be a Tag.")
-        return pattern.clone(name=name, category=category, value=value)
+        return pattern.clone(
+            name=name, category=category, value=value, capture=capture
+        )
 
 
-def _to_pattern(pattern, context="root"):
+def _select(pattern, context="root"):
     if isinstance(pattern, str):
         pattern = parse(pattern)
     if isinstance(pattern, Element):
@@ -687,13 +662,11 @@ def _to_pattern(pattern, context="root"):
     return pattern
 
 
-def to_pattern(s, env=None):
+def select(s, env=None, skip_modules=[]):
     if not isinstance(s, str):
         return s
     if env is None:
         fr = sys._getframe(1)
-        env = _find_eval_env(s, fr)
-    if env is None:
-        raise Exception(f"Could not find env for selector '{s}'")
-    pattern = _to_pattern(s)
-    return _resolve(pattern, env)
+        env = _find_eval_env(s, fr, skip=["ptera", *skip_modules])
+    pattern = _select(s)
+    return _resolve(pattern, env, count())
